#include "core.h"

Core::Core()
{
    initCore();
}

void Core::initCore()
{
    connect(KylinImageCodec::getSignalObj(),&KylinImageCodecSignals::saveMovieFinish,this,&Core::saveMovieFinish);

    m_matList = nullptr;

    m_albumModel = new QStandardItemModel;

    qRegisterMetaType<ImageShowStatus::ChangeShowSizeType>("ImageShowStatus::ChangeShowSizeType");
    qRegisterMetaType<QFileInfo>("QFileInfo");
    qRegisterMetaType<Processing::FlipWay>("Processing::FlipWay");

    m_clickBeforeStartPosition = QPoint(-1,-1);
    m_clickBeforePosition = QPoint(-1,-1);

    m_playMovieTimer = new QTimer(this);
    connect(m_playMovieTimer,&QTimer::timeout,this,&Core::playMovie);

    m_canProcess = new QTimer(this);
    m_canProcess->setSingleShot(true);
}

QString Core::initDbus(const QStringList &arguments)
{
    m_dbus = new Dbus;

    //获取注册Dbus服务是否成功
    bool connectSeccess=m_dbus->getConnectSeccess();

    //如果注册Dbus服务成功，接收命令
    if (connectSeccess) {
        connect(m_dbus,&Dbus::processingCommand,this,&Core::processingCommand);
        connect(m_dbus,&Dbus::processingFullScreen,this,&Core::processingFullscreen);
    }

    //没有参数不处理
    if (arguments.length() <2) {
        return "";
    }

    //帮助命令则打印
    if (arguments[1]=="-help" || arguments[1]=="--help" || arguments[1]=="-h") {
        for (QString &key : Variable::SUPPORT_CMD.keys()) {
            QString str = key + "   " + Variable::SUPPORT_CMD.value(key);
            QStringList list = str.split("\n");
            for (QString &tmpStr : list) {
                qInfo()<<tmpStr;
            }
        }
        exit(0);
        return "";
    }

    //如果是地址且文件存在则打开
    QString path = arguments[1];
    if (path.startsWith("file:///")) {
        path.replace("file:///","/");
    }
    if (QFileInfo::exists(path)) {
        //不是支持的文件格式忽略
        QString format =path;
        format=format.split(".").last();
        if (!Variable::SUPPORT_FORMATS.contains(format.toLower())) {
            exit(0);
            return "";
        }
        return path;
    }

    if (arguments[1] == "-api") {
        return processingApi(arguments);
    }

    //如果是命令
    if (Variable::SUPPORT_CMD.keys().contains(arguments[1])) {
        if (connectSeccess) {//如果为首个实例不响应
            return "";
        }
        m_dbus->argumentsCommand(arguments);
        return "";
    }

    qDebug()<<"参数异常";
    return "";
}

void Core::processingCommand(const QStringList &cmd)
{
    if (coreOperateTooOften()) {
        return;
    }
    qDebug()<<"响应外部命令"<<cmd;

    if (cmd[0] == "-next") {
        changeImage(NEXT_IMAGE);
        return;
    }
    if (cmd[0] == "-back") {
        changeImage(BACK_IMAGE);
        return;
    }
    if (cmd[0] == "-big") {
        changeImageShowSize(ImageShowStatus::BIG);
        return;
    }
    if (cmd[0] == "-small") {
        changeImageShowSize(ImageShowStatus::SMALL);
        return;
    }
    if (cmd[0] == "-origin") {
        changeImageShowSize(ImageShowStatus::ORIGIN);
        return;
    }
    if (cmd[0] == "-auto") {
        changeImageShowSize(ImageShowStatus::AUTO);
        return;
    }
    if (cmd[0] == "-rotate") {
        flipImage(Processing::FlipWay::clockwise);
        return;
    }
}

QString Core::processingApi(const QStringList &cmd)
{
    if (cmd.length() < 4) {
        return "";
    }

    if (cmd[2] == "-flip") {
        if (cmd.length() < 5) {
            qInfo()<<"参数不合规，使用-h查看帮助";
            return "";
        }
        m_apiCmd.append(cmd[2]);
        if (!QFileInfo::exists(cmd[3])) {
            qInfo()<<"文件不存在";
            return "";
        }
        QString format =cmd[3];
        format=format.split(".").last();
        if (!Variable::SUPPORT_FORMATS.contains(format)) {
            qInfo()<<"不支持的格式";
            return "";
        }
        m_apiCmd.append(cmd[3]);
        QString flipCmd = cmd[4];
        int hNum = flipCmd.count('h');
        hNum = hNum % 2;//保留有效操作
        int vNum = flipCmd.count('v');
        vNum = vNum % 2;
        int cNum = flipCmd.count('c');
        cNum = cNum % 4;
        QString flipCmdResult;
        for (int i = 0;i < hNum;i++) {
            flipCmdResult+="h";
        }
        for (int i = 0;i < vNum;i++) {
            flipCmdResult+="v";
        }
        for (int i = 0;i < cNum;i++) {
            flipCmdResult+="c";
        }
        m_apiCmd.append(flipCmdResult);

        if (cmd.length() > 5) {
            QString saveWay = cmd[5];
            if (saveWay.indexOf('r') > 0) {
                m_apiReplaceFile = true;
            }
        }

        m_isApi = true;
        return Variable::API_TYPE;
    }
    return "";
}

void Core::processingFullscreen()
{
    Q_EMIT fullScreen();
}

void Core::setHighLight(const QString &path)
{
    for (int i = 1;i<m_albumModel->rowCount();i++) {
        MyStandardItem * item =dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
        if (item->getPath() == path) {
            Q_EMIT changeAlbumHighLight(item->index());
            return;
        }
    }
}

void Core::openImage(QString fullPath)
{
    //1.文件路径为空；2.从外部删除文件夹下的所有图片时，传空图片列表，不展示图片，回到开始界面
    if (fullPath.isEmpty()) {
        showImage(QPixmap());
        return;
    }
    if (m_isclose == true) {
        return;
    }
    //判断图片状态
    QFlags<QFileDevice::Permission> power = QFile::permissions(fullPath);
    //判断文件所有者是否可写和是否可删除
    if (power.testFlag(QFile::WriteOwner) && File::canDel(fullPath)) {
        m_isCanDel = true;
        //发送信号给前端--可删除
        Q_EMIT canDelImage(true);
    } else {
        m_isCanDel = false;
        //发送信号给前端--不可删除
        Q_EMIT canDelImage(false);
    }

    m_willProcess.clear();
    needSave();

    MatAndFileinfo maf = File::loadImage(fullPath);
    m_openSuccess = maf.openSuccess;//判断是否打开失败-损坏
    if (!m_openSuccess) {
        //更新侧栏图标
        changeOpenIcon(Variable::g_themeStyle);
    }
    setHighLight(fullPath);//设置相册选中
    m_nowImage = Processing::converFormat(maf.mat);
    //记录状态
    changeMat(maf.mat);
    m_info = maf.info;
    m_maxFrame = maf.maxFrame;
    if (m_matList!=nullptr) {
        m_matList->clear();
    }
    m_matList = maf.matList;
    m_fps = maf.fps;
    m_nowpath = fullPath;
    //首次打开，m_backpath为空，造成相册切换
    if (m_backpath == nullptr) {
        m_backpath = m_nowpath;
    }
    //如果正在保存，则显示等待
    if (File::isSaving(fullPath)) {
        //如果此图正在保存
        m_thisImageIsSaving = true;
        showImage(QPixmap());
        return;
    }
    creatImage(-1,true);
    return;
}

void Core::showImage(const QPixmap &pix)
{
    m_origCutImage = pix.toImage().copy();

    if (!m_isCutting) {
        m_origCutImageBefore = pix.toImage().copy();
    }
    ImageAndInfo package;
    package.proportion = m_proportion;
    package.imageNumber = m_albumModel->rowCount();
    package.info = m_info;
    package.image = pix;
    m_paperFormat = m_info.suffix().toLower();
    package.openSuccess = m_openSuccess;
    if (!m_openSuccess) {
        package.imageSize = "-";
        package.colorSpace = "-";
    } else {
        package.imageSize = m_imageSize;
        package.colorSpace = m_colorSpace;
    }


    QVariant var;
    var.setValue<ImageAndInfo>(package);
    Q_EMIT openFinish(var);
}

void Core::creatImage(const int &proportion, bool defaultOpen)
{

    if (m_nowImage.isNull()) {
        return;
    }

    int defaultProportion  = 100 * m_size.width() / m_nowImage.width();
    if (m_nowImage.height() * defaultProportion / 100 > m_size.height()) {
        defaultProportion = 100 * m_size.height() / m_nowImage.height();
    }
    if (defaultOpen == false) {
        operateImage(proportion,defaultProportion);
    } else {
        defaultImage(proportion,defaultProportion);
    }

}
//默认打开，旋转等，不进行放缩操作的方式查看图片
void Core::defaultImage(int proportion, int defaultProportion)
{
    //自适应窗口大小显示
    if (proportion <= 0) {
        if (defaultProportion >= 100) {
             m_proportion = 100;
        } else {
            m_proportion  = defaultProportion;
        }
        m_tmpSize = m_nowImage.size() * m_proportion / 100;
        navigation(); //关闭导航器
        showImageOrMovie();
        return;
    }

}
//对图片尺寸进行了操作的方式查看图片
void Core::operateImage(int proportion, int defaultProportion)
{
    //计算缩放比
    if (proportion <= 0) {
        if (defaultProportion > 1000) {
            m_proportion = 1000;
        } else {
            m_proportion  = defaultProportion;
        }
    } else {
        m_proportion = proportion;
    }
    //计算缩放后尺寸
    m_tmpSize = m_nowImage.size() * m_proportion / 100;
    //极小的图缩放后尺寸低于1的不处理
    if (judgeSizeIsZero(m_tmpSize)) {
        return;
    }
    //如果显示比例大于默认比例
    if (m_proportion > defaultProportion) {
        navigation(QPoint(0,0));//显示导航器
        return;
    } else {
       navigation(); //关闭导航器
    }
    showImageOrMovie();
}

void Core::showImageOrMovie()
{

    //动画类格式循环播放
    if (m_matList!=nullptr) {
        if (m_matList->length()>1) {
            playMovie();//立即播放第一帧
            m_playMovieTimer->start(m_fps);
            return;
        }
    }
    //非动画类格式
    QPixmap pix = Processing::resizePix(m_nowImage,m_tmpSize);
    showImage(pix);
}

void Core::playMovie()
{
    if (m_matListIndex >= m_matList->length()) {
        m_matListIndex=0;
    }

    Mat mat =m_matList->at(m_matListIndex);
    QPixmap nowIndexPix = Processing::converFormat(mat);
    m_matListIndex++;

    if (m_isNavigationShow) {

        QPixmap result = Processing::localAmplification(nowIndexPix,m_tmpSize,m_startShowPoint,m_size);
        showImage(result);
        return;
    }
    QPixmap pix = Processing::resizePix(nowIndexPix,m_tmpSize);
    showImage(pix);
}

void Core::navigation(const QPoint &point)
{
    m_clickBeforeStartPosition = QPoint(-1,-1);
    if ( point.x()<0 || point.y()<0 ) {//关闭导航器
        m_clickBeforePosition = QPoint(-1,-1);
        m_isNavigationShow = false;
        Q_EMIT showNavigation(QPixmap());
        return;
    }
    m_isNavigationShow = true;
    creatNavigation();
    //记录上次放大位置
    clickNavigation();
}

void Core::clickNavigation(const QPoint &point)
{
    bool hasArg = true;
    //无参输入则使用上次的位置
    if (point==QPoint(-1,-1)) {
        hasArg=false;
    }

    //有参则记录，无参使用上一次的点
    if (hasArg) {
        m_clickBeforePosition = point;
    }

    //计算点击区域——鼠标要在高亮区域中央，且要减去导航栏窗口与图片边缘的距离
    QPoint startPoint(m_clickBeforePosition.x() - m_hightlightSize.width() / 2 - m_spaceWidth,m_clickBeforePosition.y() - m_hightlightSize.height() / 2 - m_spaceHeight);
    int right = m_navigationImage.width() - m_hightlightSize.width();//右侧边缘
    int bottom = m_navigationImage.height() - m_hightlightSize.height();//下侧边缘

    //过滤无效区域
    if (startPoint.x()<0) {
        startPoint.setX(0);
    }
    if (startPoint.y()<0) {
        startPoint.setY(0);
    }
    if (startPoint.x()>right) {
        startPoint.setX(right);
    }
    if (startPoint.y()>bottom) {
        startPoint.setY(bottom);
    }

    //有参情况下，和上次点击的有效区域一致则不处理
    if (startPoint == m_clickBeforeStartPosition && hasArg) {
        return;
    }
    m_clickBeforeStartPosition = startPoint;

    //处理导航器图片
    QImage image = Processing::pictureDeepen(m_navigationImage,m_hightlightSize,startPoint);

    //发送到导航器
    Q_EMIT showNavigation(QPixmap::fromImage(image));
    //发送到导航器设置
    Q_EMIT sendHightlightPos(QPoint(startPoint.x() +  m_spaceWidth,startPoint.y() +  m_spaceHeight) ,QPoint(startPoint.x() + m_hightlightSize.width() + m_spaceWidth,startPoint.y() + m_hightlightSize.height() + m_spaceHeight));

    //处理待显示区域
    m_startShowPoint = startPoint * m_tmpSize.width() / m_navigationImage.width();

    //如果是动图，则交给动图显示事件去处理，避免闪屏
    if (m_playMovieTimer->isActive()) {
        return;
    }
    QPixmap result = Processing::localAmplification(m_nowImage,m_tmpSize,m_startShowPoint,m_size);
    showImage(result);
}

void Core::saveMovieFinish(const QString &path)
{
    //如果收到关闭信号，保存完再退出
    if (m_shouldClose) {
        //双层判断，避免多次确认状态，提升效率
        if (File::allSaveFinish()) {
            progremExit();
        }
        return;
    }

    //如果当前播放的不是这张图
    if (m_nowpath != path) {
        return;
    }
    m_thisImageIsSaving = false;
    openImage(path);
}
//处理上一次图片没处理完，进行下一次操作的时候，更新对齐动图每一帧的操作状态
void Core::processNewLoadImage ()
{
    //m_willProcess为空，说明上一次处理完毕，不需要进行对齐更新
    if (m_willProcess.isEmpty()) {
        return;
    }
    //从未处理处开始进行对齐，m_willProcessNum为上一次处理到第几帧
    for (int i = m_willProcessNum;i<m_matList->length();i++) {
        //遍历队列，对剩下帧进行同样的操作，保持所有帧一致
        for (Processing::FlipWay &tmpWay :m_willProcess) {
            Mat mat = Processing::processingImage(Processing::flip,m_matList->at(i),QVariant(tmpWay));
            m_matList->replace(i,mat);
        }
    }
}

void Core::flipImage(const Processing::FlipWay &way)
{
    //如果此图正在保存
    if (m_thisImageIsSaving) {
        return;
    }

    m_processed = true;

    processNewLoadImage();

    //如果是动图，则批量处理
    if (m_playMovieTimer->isActive()) {
        for (int i=0;i<m_matList->length();i++) {
            Mat mat = Processing::processingImage(Processing::flip,m_matList->at(i),QVariant(way));
            m_matList->replace(i,mat);
        }
        //判断此次操作是否未加载完全，如是，记录此时的加载帧数和操作方式
        if (m_maxFrame > m_matList->length()) {
            m_willProcessNum = m_matList->length();
            m_willProcess.append(way);
        } else {
            m_willProcess.clear();
        }
        //刷新导航栏
        m_nowImage = Processing::converFormat(m_matList->first());
        creatImage(-1,true);
        return;
    }
    Mat mat = Processing::processingImage(Processing::flip,m_nowMat,QVariant(way));
    if (!mat.data) {
        return;
    }
    mat = changeMat(mat);
    m_nowImage = Processing::converFormat(mat);
    creatImage(-1,true);
}

void Core::deleteImage()
{
    //如果此图正在保存
    if (m_thisImageIsSaving) {
        return;
    }

    m_processed=false;//重置是否操作过的状态,都已经删除了，就没必要存了

    if (!m_isCanDel) {
        return;
    }
    //切换到下一张
    changeImage(NEXT_IMAGE);

    //从队列中去除
    deleteAlbumItem(m_backpath);
    //更新相册列表
    Q_EMIT updateSideNum(m_albumModel->rowCount());

    setHighLight(m_nowpath);

    //删除后队列中无图片，返回状态
    if(m_albumModel->rowCount() == 1){
        navigation();//关闭导航器
        showImage(QPixmap());//发送状态
        return;
    }
}

void Core::setAsBackground()
{
    //设置为背景图
    if (QGSettings::isSchemaInstalled(SET_BACKGROUND_PICTURE_GSETTINGS_PATH)) {
        QGSettings *background = new QGSettings(SET_BACKGROUND_PICTURE_GSETTINGS_PATH);
        QStringList keyList = background->keys();
        if (keyList.contains(SET_BACKGROUND_PICTURE_GSETTINGS_NAME)) {
            QString nowPath =  background->get(SET_BACKGROUND_PICTURE_GSETTINGS_NAME).toString();
            if (nowPath == m_nowpath) {
                background->set(SET_BACKGROUND_PICTURE_GSETTINGS_NAME,QString(""));
            }
            background->set(SET_BACKGROUND_PICTURE_GSETTINGS_NAME,m_nowpath);
        }
        background->deleteLater();
    }
}

void Core::openInfile()
{
    QDBusInterface interface("org.freedesktop.FileManager1",
                             "/org/freedesktop/FileManager1",
                             "org.freedesktop.FileManager1",
                             QDBusConnection::sessionBus());
    QStringList items;
    items.push_back(m_nowpath);
    QString startUpId = "";
    interface.call("ShowItems",items,startUpId);
}

void Core::changeOpenIcon(QString theme)
{
    Variable::g_themeStyle = theme;

    //将第一张图设置为打开按钮
    if (m_item0 == nullptr) {
        m_item0 = new MyStandardItem;
        m_item0->setDragEnabled(false);
        m_item0->setSizeHint(Variable::ALBUM_IMAGE_SIZE);
        m_item0->setToolTip(tr("Add"));
        m_albumModel->insertRow(0,m_item0);//插入model
    }
    if ("ukui-dark" == theme || "ukui-black" == theme) {
        if (m_item0 != nullptr) {
            m_item0->setIcon(m_item0Black);//用来保存地址路径
            m_openImage = QPixmap(":/res/res/damaged_imgB.png");
        }
    } else {
        if (m_item0 != nullptr) {
            m_item0->setIcon(m_item0White);//用来保存地址路径
            m_openImage = QPixmap(":/res/res/damaged_img.png");
        }
    }
    if (!m_openSuccess) {
        showImage(m_openImage);
    }
    for (int i = 0;i<m_albumModel->rowCount();i++) {
        MyStandardItem * item =dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
        if (item == nullptr) {
            continue;
        }
        if (Variable::g_damagedImage.contains(item->getPath())) {
            item->setIcon(m_openImage);
        }
    }
}

void Core::close()
{

    //如果已经触发过关闭事件不响应
    if (m_shouldClose) {
        return;
    }
    //如果正在裁剪,且还未保存--暂时逻辑，标题栏已隐藏，暂时不用处理
//    if (m_isCutting && m_isCutSaved) {
//        Q_EMIT isCutNotSave();
//    }
    //如果没有操作且有正在保存的动图直接退出
    if (!m_processed) {
        if (!File::allSaveFinish()) {
            if (m_playMovieTimer->isActive()) {
                m_playMovieTimer->stop();
            }
            m_shouldClose = true;
            showImage(QPixmap());
            return;
        }
        progremExit();
        return;
    }
    //如果不是作为外部API调用则默认覆盖保存
    if (!m_isApi) {
        m_apiReplaceFile = true;
    }
    m_isclose = true;
    //如果正在播放动图，则停止
    if (m_playMovieTimer->isActive()) {
        m_playMovieTimer->stop();
        showImage(QPixmap());
        //判断没有加载完全部时，进行阻塞
        while (m_maxFrame > m_matList->length()) {
            QThread::usleep(10);
        }
        processNewLoadImage();
        //保存动图
        File::saveImage(m_matList,m_fps,m_nowpath,m_apiReplaceFile);
        m_shouldClose = true;
    } else {
        //保存图片
        File::saveImage(m_nowMat,m_nowpath,m_apiReplaceFile);
        QString suffix = QFileInfo(m_nowpath).suffix().toLower();
        if (suffix == "apng" || suffix == "png" || suffix == "gif") {
            if (m_matList != nullptr) {
                if (m_matList->length() > 1) {
                    m_shouldClose = true;
                    return;
                }
            }
            //解决git静态图无法保存的问题，已测试，界面可正常关掉，png和apng可正常保存
            if (suffix == "gif") {
                m_shouldClose = true;
                return;
            }
        }
        progremExit();
    }
}

QStandardItemModel *Core::getAlbumModel()
{
    return m_albumModel;
}

void Core::changeImage(const int &type)
{
    //如果错误不处理
    if (type == ERROR_IMAGE) {
        Q_EMIT updateSideNum(m_albumModel->rowCount());
        return;
    }

    //如果图片队列小于1，不处理。小于2时，相册文件清空时，会一直显示当前图片
    if (m_albumModel->rowCount()<2) {
        m_backpath = m_nowpath;
        return;
    }
    if (m_isclose == true) {
        return;
    }
    needSave();

    if (type == NEXT_IMAGE) {
        QString key = nextImagePath(m_nowpath);
        changeImageType(key);
        openImage(m_nowpath);
        loadOnChange(m_nowpath);
        return;
    }
    if (type == BACK_IMAGE) {
        QString key = backImagePath(m_nowpath);
        changeImageType(key);
        openImage(m_nowpath);
        loadOnChange(m_nowpath);
        return;
    }

    //点击相册切换图片
    MyStandardItem *item = dynamic_cast<MyStandardItem *>(m_albumModel->item(type));
    changeImageType(item->getPath());
    reloadImage(m_albumModel->indexFromItem(item));
    openImage(m_nowpath);
}

void Core::changeImageFromClick(QModelIndex modelIndex)
{
    MyStandardItem *item = dynamic_cast<MyStandardItem *>(m_albumModel->itemFromIndex(modelIndex));
    if (item == nullptr) {
        changeImage(modelIndex.row());
        return;
    }
    if (item->getPath() == m_nowpath) {
        return;
    }
    //判断是按钮点击，则打开图片
    if (modelIndex.row() == 0) {
        Q_EMIT openFromAlbum();
        return;
    }
    changeImage(modelIndex.row());
//    openImage(item->getPath());
}

void Core::changeWidgetSize(const QSize &size)
{
    m_size = size;
    creatImage(-1,true);
}

void Core::changeImageShowSize(ImageShowStatus::ChangeShowSizeType type)
{
    //如果此图正在保存
    if (m_thisImageIsSaving) {
        return;
    }

    int resizeKey = Variable::RESIZE_KEY;
    int tmpProportion = 0;
    switch (type) {

    ///----------------------------放大图片----------------------------
    case ImageShowStatus::BIG:
        if (m_proportion == Variable::RESIZE_KEY_MAX) {//等于临界值
            return;
        }
        if (m_proportion + resizeKey > Variable::RESIZE_KEY_MAX) {//将要超出临界值
            tmpProportion = Variable::RESIZE_KEY_MAX;
            creatImage(tmpProportion,false);
            return;
        }
        if (m_proportion % resizeKey != 0) { //不能整除
            tmpProportion = (m_proportion/resizeKey + 1)*resizeKey;
            creatImage(tmpProportion,false);
            return;
        }
        tmpProportion = m_proportion + resizeKey;
        creatImage(tmpProportion,false);
        return;

        ///----------------------------缩小图片----------------------------
    case ImageShowStatus::SMALL:
        if (m_proportion == Variable::RESIZE_KEY_MIN) {//等于临界值
            return;
        }
        if (m_proportion - resizeKey < Variable::RESIZE_KEY_MIN) {//将要超出临界值
            tmpProportion = Variable::RESIZE_KEY_MIN;
            creatImage(tmpProportion,false);
            return;
        }
        if (m_proportion % resizeKey != 0) { //不能整除
            tmpProportion = (m_proportion/resizeKey)*resizeKey;
            creatImage(tmpProportion,false);
            return;
        }
        tmpProportion = m_proportion - resizeKey;
        creatImage(tmpProportion,false);
        return;

        ///----------------------------查看原图----------------------------
    case ImageShowStatus::ORIGIN:
        creatImage(100,false);
        return;

        ///----------------------------查看自适应图----------------------------
    case ImageShowStatus::AUTO:
        creatImage(-1,false);
        return;

    default:
        qDebug()<<"代码错误，请检查";
        break;
    }
}

void Core::changeImageType(QString path)
{
    m_matListIndex = 0;
    if (path == "") {//回滚
        deleteAlbumItem(m_nowpath);
        if (m_albumModel->rowCount() == 1) {
            m_backpath = m_nowpath;
            m_nowpath = "";
            return;
        }
        m_nowpath = m_backpath;
        return;
    }
    m_backpath = m_nowpath;
    m_nowpath = path;
}

Mat Core::changeMat(Mat mat)
{
    m_backMat = m_nowMat;
    m_nowMat = mat;
    m_imageSize = QString::number(m_nowImage.width())+"x"+QString::number(m_nowImage.height());
    switch (m_nowMat.type()) {
    case CV_8UC4:
        m_colorSpace = "RGBA";
        break;
    case CV_8UC3:
        m_colorSpace = "RGB";
        break;
    case CV_8UC1:
        m_colorSpace = "GRAY";
        break;
    }
    //gif走的是apng流程，所以这里要单独判断
    if (QFileInfo(m_nowpath).suffix().toLower() == "gif") {
        m_colorSpace = "RGB";
    }
    return m_nowMat;
}

void Core::creatNavigation()
{
    //导航栏背景
    QSize navigationSize = Variable::NAVIGATION_SIZE;
    m_navigationImage = Processing::resizePix(m_nowImage,navigationSize).toImage();

    //记录空白区域
    m_spaceWidth = (navigationSize.width()-m_navigationImage.width())/2;
    m_spaceHeight = (navigationSize.height()-m_navigationImage.height())/2;

    //待显示图
    m_tmpSize = m_nowImage.size() * m_proportion / 100;

    //高亮区域大小
    m_hightlightSize.setWidth(m_navigationImage.width() * m_size.width() /  m_tmpSize.width());
    m_hightlightSize.setHeight(m_navigationImage.height() * m_size.height() /  m_tmpSize.height());
    if (m_hightlightSize.width()>m_navigationImage.width()) {
        m_hightlightSize.setWidth(m_navigationImage.width());
    }
    if (m_hightlightSize.height()>m_navigationImage.height()) {
        m_hightlightSize.setHeight(m_navigationImage.height());
    }
}

void Core::deleteAlbumItem(const QString &path)
{
    if (path == "") {
        return;
    }
    for (int i = 1;i<m_albumModel->rowCount();i++) {
        MyStandardItem * item = dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
        if (item->getPath() == path) {
            if (m_nowpath == "" && m_backpath == "") {
                m_backpath = dynamic_cast<MyStandardItem *>(m_albumModel->item(i-1))->getPath();
            }
            m_albumModel->removeRow(i);
            return;
        }
    }
}

Enums::ChamgeImageType Core::nextOrBack(const QString &oldPath, const QString &newPath)
{

    for (int i = 1;i<m_albumModel->rowCount();i++) {
        MyStandardItem * item = dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
        if (item->getPath() == oldPath) {
            return NEXT_IMAGE;
        }
        if (item->getPath() == newPath) {
            return BACK_IMAGE;
        }
    }
    qDebug()<<"判断切换状态错误";
    return ERROR_IMAGE;
}

Enums::RenameState Core::successOrFail(const QString &oldPath, const QString &newPath)
{
    QFile file(oldPath);
    bool ok = file.rename(oldPath,newPath);
    //改名
    if (ok) {
        QFileInfo fileToCore(newPath);
        QString fileNewName = fileToCore.absoluteFilePath();
        //更新相册，m_nowpath，m_info信息为重命名后的图片
        m_nowpath = fileNewName;
        m_info = fileToCore;
        for (int i = 1;i<m_albumModel->rowCount();i++) {
            MyStandardItem * item = dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
            if (item->getPath() == oldPath) {
                item->setPath(fileNewName);
            }
        }
        return SUCCESS;
    } else {
        if (!file.exists()) {
            qDebug()<<"文件不存在";
            return NOT_EXITS;
        }
        if (QFile::exists(newPath)) {
            qDebug()<<"文件同名";
            return SAME_NAME;
        }
        return UNKNOWN_ERROR;
    }
}

QString Core::nextImagePath(const QString &oldPath)
{
    //相册中没有文件时，不进行处理
    if (m_albumModel->rowCount()<2) {
        return "";
    }
    MyStandardItem * item =nullptr;
    //最后一张切下一张时，回到队列开头
    item = dynamic_cast<MyStandardItem *>(m_albumModel->item(m_albumModel->rowCount()-1));
    if (item->getPath() == oldPath) {
        item = dynamic_cast<MyStandardItem *>(m_albumModel->item(1));
        return item->getPath();
    }

    bool tmp = false;
    for (int i = 1;i<m_albumModel->rowCount();i++) {
        item = dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
        if (tmp) {
            return item->getPath();
        }
        if (item->getPath() == oldPath) {
            tmp = true;
        }
    }
    return "";
}

QString Core::backImagePath(const QString &oldPath)
{
    //相册中没有文件时，不进行处理
    if (m_albumModel->rowCount()<2) {
        return "";
    }
    MyStandardItem * item =nullptr;
    //第一张切上一张时，回到队列结尾
    item = dynamic_cast<MyStandardItem *>(m_albumModel->item(1));
    if (item->getPath() == oldPath) {
        item = dynamic_cast<MyStandardItem *>(m_albumModel->item(m_albumModel->rowCount()-1));
        return item->getPath();
    }

    QString tmp = "";
    for (int i = 1;i<m_albumModel->rowCount();i++) {
        item = dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
        QString tmp2 = item->getPath();
        if (tmp2 == oldPath) {
            return tmp;
        }
        tmp = tmp2;
    }
    return "";
}

void Core::progremExit()
{
    this->deleteLater();
    Q_EMIT coreProgremExit();
}

void Core::albumLoadFinish(QVariant var)
{
    ImageAndInfo package = var.value<ImageAndInfo>();
    QString fileName = package.info.absoluteFilePath();
    for (int i = 1;i<m_albumModel->rowCount();i++) {
        MyStandardItem * item = dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
        if (item->getPath() == fileName) {
            if (package.image.isNull()) {
                m_albumModel->removeRow(i);
                return;
            }
            m_albumModel->item(i)->setIcon(package.image);
            return;
        }
    }
}
//重命名
void Core::toCoreChangeName(QString oldName, QString newName)
{
    //存原图后缀
    QFileInfo file(oldName);
    QString oldSuffix = file.suffix();
    oldSuffix = "." + oldSuffix;
    //形成完整路径
    QString path = file.absolutePath();
    QString newPath = path + "/" + newName;
    newPath = newPath + oldSuffix;
    //判断是否重命名成功，并返回重命名结果
    RenameState type = successOrFail(oldName,newPath);
    //重命名返回结果---给前端发信号改名或错误提示
    if (type == SUCCESS) {//重命名成功
        QFileInfo fileToCore(newPath);
        Q_EMIT renameResult(0,fileToCore);
    } else if (type == NOT_EXITS){//文件不存在
        Q_EMIT renameResult(-1,file);
    } else if (type == SAME_NAME) {//文件同名
        Q_EMIT renameResult(-2,file);
    } else if (type == NO_PEIMISSION) {//没有权限----暂时算为其他未知错误中，待以后解决
        Q_EMIT renameResult(-3,file);
    } else {//其他未知错误
        Q_EMIT renameResult(-4,file);
    }
}

void Core::toPrintImage(QPrinter *printer)
{
    //图片使用原图进行打印
    QImage img = m_nowImage.toImage();
    QPainter painter(printer);
    QRect rect=painter.viewport();
    QSize size=img.size();
    size.scale(rect.size(),Qt::KeepAspectRatio);
    painter.setViewport(rect.x(),rect.y(),size.width(),size.height());
    painter.setWindow(img.rect());
    painter.drawImage(0,0,img);
}
//打开失败，判断是否为拖拽打开失败--是的话，不进行切换
bool Core::openFail(QString fullPath)
{
    //注--此函数只有在打开失败的时候才进来，不影响正常的图片
    QStringList m_pathList;
    //遍历整个相册
    for (int i = 1;i<m_albumModel->rowCount();i++) {
        MyStandardItem * item =dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
        m_pathList.append(item->getPath());
    }
    //如果不包含当前路径，则说明是外部拖入
    if (!m_pathList.contains(fullPath)) {
        return true;
    } else {
        return false;
    }
}
//裁剪，处理显示的图片
void Core::cutImage()
{
    m_isCutting = true;
    QImage image =  Processing::pictureCutDeepen(m_origCutImage,m_origCutImage.size());
    showImage(QPixmap::fromImage(image));
}
//裁剪区域
void Core::cutRegion(const QPoint &point)
{
    //起始点变化
    if (m_cutBeforeStartPosition != m_cutStartPostionSave && m_cutStartPostionSave.x() != -1 && m_cutStartPostionSave.y() !=-1) {
        //多个起点
        return;
    }
    //判断是否是边界
    QPoint changePoint = boundingSet(point,m_origCutImageBefore.width(),m_origCutImageBefore.height());
    if (m_backPointRelease.x() != -1 && m_backPointRelease.y() != -1) {

        m_backPointRelease = boundingSet(m_backPointRelease,m_origCutImageBefore.width(),m_origCutImageBefore.height());
    }
    //判断只是点击一下，不进行绘制
    if (changePoint == m_cutBeforeStartPosition) {
        return;
    }
    //高亮区域
    m_cuthightlightSize = QSize(abs(point.x() - m_cutBeforeStartPosition.x()),abs(point.y() - m_cutBeforeStartPosition.y()));
    //处理待显示的图片
    QImage image =  Processing::pictureCutImageDeepen(m_origCutImage,m_cuthightlightSize,m_cutBeforeStartPosition,m_origCutImageBefore,m_cuthightlightSizeBefore,changePoint);
    //保存上一个区域
    m_cuthightlightSizeBefore = m_cuthightlightSize;
    //保存上一个起始点
    m_cutStartPostionSave = m_cutBeforeStartPosition;
    m_isReleasePaint = false;//非释放鼠标的时候要重置
    m_isCutSaved = true;//开始裁剪当并未保存
    //重置上一个点
    m_backPointRelease.setX(-1);
    m_backPointRelease.setY(-1);
    showImage(QPixmap::fromImage(image));
    //将当前画完框的图像存起来
    m_cutImage = image;
}
//记录起始点
void Core::startChange(const QPoint &point)
{
    m_cutBeforeStartPosition = point;
}
//释放鼠标
void Core::realsePos(const QPoint &point)
{
    //判断裁剪框已经画好了，就不需要再次重绘
    if (m_hasCutBox) {
        return;
    }
    m_isReleasePaint = false;//释放鼠标的时候要重置
    //重置上一个点
    m_backPointRelease.setX(-1);
    m_backPointRelease.setY(-1);
    QImage image =  Processing::pictureRelease(m_cutImage,m_origCutImageBefore);
    showImage(QPixmap::fromImage(image));
    m_startP = Processing::releaseStart();
    m_endP = Processing::releaseEnd();
    Q_EMIT releaseRegion(m_startP,m_endP);

}

void Core::releaseCutChange(int type, const QPoint &point)
{
    //鼠标没有释放，状态不能改变，保证鼠标一直是当前这个状态
    if (m_isReleasePaint) {
        type = m_backType;
        //给前端发信号，设置鼠标状态
        Q_EMIT returnMouseType(type);
    }
    //边界判断
    QPoint changPoint = boundingSet(point,m_origCutImageBefore.width(),m_origCutImageBefore.height());

    if (m_backPointRelease.x() != -1 && m_backPointRelease.y() != -1) {

        m_backPointRelease = boundingSet(m_backPointRelease,m_origCutImageBefore.width(),m_origCutImageBefore.height());
    }
    QImage image =  Processing::pictureCutChange(type,m_backType, m_cutImage,m_startP,m_endP,changPoint,m_backPointRelease,m_origCutImageBefore);
    m_isReleasePaint = true;
    showImage(QPixmap::fromImage(image));
    m_cutImage = image;
    m_backType = type;
    m_backPointRelease = point;
}

void Core::exitCut()
{
    showImage(QPixmap::fromImage(m_origCutImageBefore));
    m_origCutImage = m_origCutImageBefore.copy();
    //状态重置
    cutFinishReset();
}
//裁剪后保存
void Core::needSaveCutImage(int proportion)
{
    QString savePath;
    QPointF startPosition =QPointF (static_cast<double> (m_startP.x())/(static_cast<double> (proportion)*0.01) ,static_cast<double> (m_startP.y())/(0.01*static_cast<double> (proportion))) ;
    QPointF endPosition = QPointF (static_cast<double> (m_endP.x())/(static_cast<double> (proportion)*0.01),static_cast<double> (m_endP.y())/(static_cast<double> (proportion)*0.01)) ;
    int imageWidth = endPosition.x() - startPosition.x();
    int imageHeight = endPosition.y() - startPosition.y();
    //确定最终裁剪区域
    CutRegion cutRegion = cutRegionSelection(startPosition,QSize(imageWidth,imageHeight),m_nowImage);
    cv::Mat tempMat = m_nowMat(cv::Rect(cutRegion.startPostion.x(), cutRegion.startPostion.y(), cutRegion.width, cutRegion.height)).clone();
    Q_EMIT cutSaveFinish();
    //状态重置
    cutFinishReset();
    if (File::saveImage(tempMat,m_nowpath,false)) {
        openSaveImage();
    } else {
        qDebug()<<"裁剪失败";
    }
}
//确定最终的裁剪区域，返回起始点和裁剪宽度信息
CutRegion Core::cutRegionSelection(QPointF startPos, QSize cutSize, QPixmap nowImage)
{
    CutRegion cutRegion;
    //确定起始点是否已经是图片内最大的点或超过最大的点，起始点不能是边界
    if (startPos.x() >= nowImage.width()) {
        //防止越界
        startPos.setX(nowImage.width() - 1);
    }
    if (startPos.y() >= nowImage.height()) {
        //防止越界
        startPos.setY(nowImage.height() - 1);
    }
    //判断当前裁剪区域是否超过图片区域
    if (startPos.x() + cutSize.width() > m_nowImage.width()) {
        cutSize.setWidth(m_nowImage.width() - startPos.x()) ;
    }
    if (startPos.y() + cutSize.height() > m_nowImage.height()) {
        cutSize.setHeight(m_nowImage.height() - startPos.y());
    }
    //裁剪区域的宽度和高度不能小于等于0
    if (cutSize.width() <= 0) {
        cutSize.setWidth(CutMinValue::minValue);
    }
    if (cutSize.height() <= 0) {
        cutSize.setHeight(CutMinValue::minValue);
    }
    //存到结构体
    cutRegion.startPostion = startPos;
    cutRegion.width = cutSize.width();
    cutRegion.height = cutSize.height();
    return cutRegion;
}

//另存为
void Core::needSaveAsImage(QString path)
{
    if (File::saveImage(m_nowMat,QString(path))) {
        openSaveImage();
    } else {
        //另存为失败
        qDebug()<<"另存为失败";
    }

}

//处理边界问题
QPoint Core::boundingSet(QPoint point, int xMax, int yMax)
{
    if (point.x() > xMax ) {
        point.setX(xMax );
    }
    if (point.y() > yMax) {
        point.setY(yMax);
    }
    if (point.x() < 0) {
        point.setX(0);
    }
    if (point.y() < 0) {
        point.setY(0);
    }
    return point;
}

void Core::cutFinishReset()
{
    //状态重置
    m_isCutting = false;
    m_isCutSaved = false;
    m_isReleasePaint = false;
    m_hasCutBox = false;
    m_backPointRelease = QPoint(-1,-1);
    m_cutStartPostionSave = QPoint (-1,-1);//保存前一个裁剪的起始点
    m_cuthightlightSizeBefore = QSize(0,0);//高亮区域大小-上一次的
    m_cutspaceWidth = 0;//导航栏窗口与缩略图左边缘距离
    m_cutspaceHeight = 0;//导航栏窗口与缩略图上边缘距离
}

void Core::openSaveImage()
{
    //裁剪和另存为之后打开保存的图片
    QString savePath = File::savePath();
    QFileInfo file(savePath);
    if (file.exists()) {
        findAllImageFromDir(savePath);
    } else {
        //给前端发信号处理-后续处理
        qDebug()<<"保存图片打开失败"<<savePath;
    }
}
//切换图片时加载-拿QModelIndex
void Core::loadOnChange(QString path)
{
    for (int i = 1;i<m_albumModel->rowCount();i++) {
        MyStandardItem * item = dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
        if (item->getPath() == path) {
            reloadImage(m_albumModel->indexFromItem(item));
        }
    }
}

bool Core::judgeSizeIsZero(QSize size)
{
    if (size.width() * size.height() == 0) {
        return true;
    }
    return false;
}


bool Core::apiFunction()
{
    if (!m_isApi) {
       return false;
    }
    QString fullPath = m_apiCmd[1];
    openImage(fullPath);
    //翻转
    if (m_apiCmd[0]=="-flip") {
        QString cmd = m_apiCmd[2];
        for (QChar &ch : cmd) {
            if (ch == 'h') {
                flipImage(Processing::FlipWay::horizontal);
            } else if (ch == 'v') {
                flipImage(Processing::FlipWay::vertical);
            } else if (ch == 'c') {
                flipImage(Processing::FlipWay::clockwise);
            }
        }
        close();
    }
    return true;
}
//判断是不是一个文件夹路径
bool Core::isSamePath(QString path)
{
    QString pathDir = QFileInfo(path).absolutePath();//文件夹
    QStringList pathlist;
    //此处不能-1，会造成只有两张图片时，无法寻找第一张图片的路径，重复加载此路径下的图片
    int num = m_albumModel->rowCount();
    if (num > 0) {
        for (int i = 0 ; i < num;i++) {
            MyStandardItem * item = dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
            QFileInfo info(item->getPath());
            pathlist.append(info.absolutePath());
          }
        if (pathlist.contains(pathDir)) {
            return true;
        } else {
            return false;
        }
    } else {
        return false;
    }
}
//判断统一文件夹下有不同图片时，需要在相册显示
QStringList Core::diffPath(QStringList image)
{
    QStringList pathList;//存相册的图片路径
    QStringList diffImages;//存不同图片
    int num = m_albumModel->rowCount();
    if (num > 0) {
        for (int i = 0 ; i < num ;i++) {
            MyStandardItem * item = dynamic_cast<MyStandardItem *>(m_albumModel->item(i));
            QFileInfo info(item->getPath());
            if (info.exists()) {
                pathList.append(info.fileName());
            }
        }
    } else {
        return diffImages;
    }
    //遍历image列表
    for (int j = 0 ; j < image.length() ; j++) {
        //判断，相册文件列表中无此时文件路径，存入diffImages中
        if (!pathList.contains(image.at(j))) {
            diffImages.append(image.at(j));
        }
    }
    return diffImages;

}

void Core::needSave()
{
    //解决侧栏切换和点击按钮打开图片时，上一张操作过的图片无法保存的问题
    m_thisImageIsSaving = false;

    //如果正在播放动图，则停止
    if (m_playMovieTimer->isActive()) {
        m_playMovieTimer->stop();
        //保存动图
        if (m_processed) {
            //判断没有加载完全部时，进行阻塞
            while (m_maxFrame > m_matList->length()) {
                QThread::usleep(10);
            }
            processNewLoadImage();
            File::saveImage(m_matList,m_fps,m_nowpath);
        }
    } else {
        //保存图片
        if (m_processed) {
            File::saveImage(m_nowMat,m_nowpath);
        }
    }
    m_processed=false;//重置是否操作过的状态
}


void Core::findAllImageFromDir(QString fullPath)
{
    if (apiFunction()) {
        return;
    }
    QFileInfo info(fullPath);
    QString path = info.absolutePath();//转成绝对路径
    QString filepath = info.absoluteFilePath();//转成绝对路径
    QDir dir(path);//实例化目录对象
    QStringList nameFilters;//格式过滤
    for (const QString &format : Variable::SUPPORT_FORMATS) {
        nameFilters<<"*."+format;//构造格式过滤列表
    }
    QStringList images = dir.entryList(nameFilters, QDir::Files|QDir::Readable, QDir::Name);//获取所有支持的图片
    //    QStringList images = dir.entryList(nameFilters, QDir::Files|QDir::Writable|QDir::Hidden, QDir::Name);//获取所有支持的图片

    //判断是同一文件夹下的图片，过滤掉相同的图片，加载文件夹下其他图片
    if (isSamePath(fullPath)) {
        images = diffPath(images);
    }
    QString tmpFullPath;
    //先把完整路径存起来，方便对比
    for (int i = 0; i < images.length(); i++) {
        tmpFullPath = path+"/"+images.at(i);
        m_allImageList.insert(i,tmpFullPath);
    }
    creatModule(path,images);

    openImage(filepath);
    //不论从什么位置开始都可以加载当前页图片
    loadOnChange(fullPath);

//    loadAlbum(tmpFullPath,m_allImageList);
}
//创建相册model
void Core::creatModule(const QString &path,QStringList list)
{
    //将所有图片存入队列
    int i = 1;

    for (QString &filename : list) {
        QString tmpFullPath = path+"/"+filename;
        //构造item
        MyStandardItem * item  = new MyStandardItem;
        item->setDragEnabled(false);
        item->setSizeHint(Variable::ALBUM_IMAGE_SIZE);
        item->setIcon(m_defultPixmap);

        item->setPath(tmpFullPath);//用来保存地址路径
        m_albumModel->insertRow(i,item);//插入model
        i++;
    }
}
//加载相册缩略图
void Core::loadAlbum(const QString &path, QStringList list, unsigned int page, unsigned int maxPerPage)
{
    //传入分母不能为0
    if (!maxPerPage) {
        return;
    }

    //传入页数不能大于最大页数
    unsigned int maxPage = list.length() / maxPerPage;
    if (list.length() % maxPerPage != 0) {
        maxPage++;
    }
    if (page > maxPage) {
        return;
    }

    //加载缩略图
    unsigned int cutoffPoint = (page + 1) * maxPerPage;
    if (cutoffPoint > list.length()) {
        cutoffPoint = list.length();
    }
    //拿当前的页数
    m_currentPage = page;

    for (unsigned int i = page *  maxPerPage; i < cutoffPoint; i++) {
        QString tmpFullPath = list.at(i);
        //存完整路径
        QFileInfo file(tmpFullPath);
        //判断有没有被外部删除
        if (file.exists()) {
            if (!m_loadedImage.contains(tmpFullPath)) {
                m_loadedImage.append(tmpFullPath);
            }
            AlbumThumbnail* thread= new AlbumThumbnail(tmpFullPath);
            connect(thread,&AlbumThumbnail::finished,thread,&AlbumThumbnail::deleteLater);
            connect(thread,&AlbumThumbnail::albumFinish,this,&Core::albumLoadFinish);
            thread->start();
        } else {
            //触发删除-外部删除文件的情况，加载的时候需要删掉对应的item
//            deleteAlbumItem(tmpFullPath);
        }
    }
}
//再加载图片
void Core::reloadImage(QModelIndex modelIndex)
{
    MyStandardItem *item = dynamic_cast<MyStandardItem *>(m_albumModel->itemFromIndex(modelIndex));
    if (item == nullptr) {
        return;
    }
    //判断没有被加载过
    if (!m_loadedImage.contains(item->getPath())) {
        //遍历所有图片
        for (int i = 0; i < m_allImageList.length(); i++ ) {
            if (item->getPath() == m_allImageList.at(i)) {
                loadAlbum( m_allImageList.at(i/Variable::LOADIMAGE_NUM * Variable::LOADIMAGE_NUM ),
                           m_allImageList,i/Variable::LOADIMAGE_NUM,Variable::LOADIMAGE_NUM);
            }
        }
    } else {
        //处理图片个数刚好多一点，且又没有取到item的时候，就到底了
        //判断最后一页图片没有被全部加载，需要将剩下的几张图片一起加载进来
        if (m_allImageList.length() > 0) {
            //说明还有几个被夹在后面没有加载
            if ((m_loadedImage.length() < m_allImageList.length()) &&
                    (m_currentPage + 1 == m_allImageList.length()/Variable::LOADIMAGE_NUM)) {
                for (int j = m_loadedImage.length(); j< m_allImageList.length(); j++) {
                    if (!m_loadedImage.contains(m_allImageList.at(j ))) {
                        loadAlbum(m_allImageList.at((m_currentPage + 1) * Variable::LOADIMAGE_NUM),
                                  m_allImageList,m_currentPage + 1,Variable::LOADIMAGE_NUM);
                        return;
                    }
                }
            }
        }
    }
}
bool Core::coreOperateTooOften()
{
    if (m_canProcess->isActive()) {
        return true;
    }
    m_canProcess->start(Variable::REFRESH_RATE);//刷新间隔
    return false;
}

void MyStandardItem::setPath(const QString &path)
{
    m_path = path;
}

QString MyStandardItem::getPath()
{
    return m_path;
}
